@@ -22,9 +22,10 @@ use warnings;
 no warnings;
 use vars qw($VERSION);
 
-$VERSION = '2.05_01';
+$VERSION = '2.05_02';
 
 use Carp;
+use File::Basename qw(dirname);
 use File::Spec;
 use Scalar::Util qw(blessed);
 
@@ -197,7 +198,7 @@ configuration values.
 =cut
 
 sub init
-	{	
+	{
 	my( $self, $config, %params ) = @_;
 
 	$self->_set_defaults( %params );
@@ -233,7 +234,7 @@ sub _set_defaults
 			quiet          => 0,
 			devnull        => File::Spec->devnull,
 			ignore_prereqs => '',
-			
+
 			%params,
 		   };
 
@@ -245,13 +246,13 @@ sub _set_defaults
 	$self->set_perl( $^X );
 	$self->add_a_perl( $^X );
 
-	# setup for Module::Build. This is a kludge. There isn't a 
+	# setup for Module::Build. This is a kludge. There isn't a
 	# programmatic interface to Makemaker, and I don't want to
 	# treat Makemaker and Module::Build differently. I'm stuck
 	# with a fancy shell script.
 	if( -e 'Build.PL' )
 		{
-		$self->{'make'}        = './Build';
+		$self->{'make'}        = File::Spec->catfile(qw{. Build});
 		$self->{'Makefile.PL'} = 'Build.PL';
 		$self->{'Makefile'}    = '_build';
 		}
@@ -295,17 +296,17 @@ sub _process_configuration
 		[ qw(Makefile    makefile)    ],
 		[ qw(make        make)        ],
 		);
-		
+
 	foreach my $pair ( @pairs )
 		{
 		my( $key, $config ) = @$pair;
-		
+
 		$self->{$key} = $self->config->get($config)
 			if $self->config->exists($config);
 		}
 	}
-	
-	
+
+
 	my @required = qw(  );
 
 	my $ok = 1;
@@ -381,7 +382,7 @@ sub load_mixin
 	return 1 if $self->mixin_loaded( $module );
 
 	no warnings 'redefine';
-	
+
 	eval "use $module";
 
 	$self->_die( "Could not load [$module]! $@" ) if $@;
@@ -436,7 +437,7 @@ sub get_web_user_agent { $_[0]->{web_user_agent} || $_[0]->setup_web_user_agent
 Create a web user agent and store it in the object. Get a reference to it
 by saving the result or calling C<get_web_user_agent>. If you call this
 method again, it replaces the internal web user agent (and anything you've
-done to it since you first set it up). It passes any arguments to the 
+done to it since you first set it up). It passes any arguments to the
 constructor of the user agent class.
 
 It calls C<web_user_agent_name> to set the initial agent name, and sets
@@ -444,17 +445,17 @@ up an empty, in-memory cookie jar.
 
 =cut
 
-sub setup_web_user_agent 
+sub setup_web_user_agent
 	{
 	my $self = shift;
-	
+
 	my $class = $self->web_user_agent_class;
 	my $rc = eval { eval "require $class" };
 	unless( $rc ) { $self->_die( "Could not load $class: $@" ) };
-	
-	my $ua = $self->web_user_agent_class->new( 
+
+	my $ua = $self->web_user_agent_class->new(
 		agent => $self->web_user_agent_name,
-		@_ 
+		@_
 		);
 
 	$ua->cookie_jar( {} );
@@ -472,7 +473,7 @@ sub web_user_agent_class { 'LWP::UserAgent' }
 
 =item web_user_agent_name
 
-The user agent name to use with web requests. Let's just call that 
+The user agent name to use with web requests. Let's just call that
 C<Mozilla> for now.
 
 =cut
@@ -601,6 +602,14 @@ sub add_a_perl
 	unless( -x $path )
 		{
 		$self->_warn( "$path is not executable" );
+		if( $path =~ m/[*?[]/ && $self->config->allow_glob_in_perls )
+			{
+			$self->add_a_perl( $_ ) for glob $path;
+			}
+		else
+			{
+			$self->_warn( "$path is not executable" );
+			}
 		return;
 		}
 
@@ -841,8 +850,28 @@ sub test
 
 	my $tests = $self->run( "$self->{make} test 2>&1" );
 
-	$self->_die( "\nERROR: Tests failed!\n$tests\n\nAborting release\n" )
-		    unless $tests =~ /All tests successful/;
+	unless ($tests =~ m/All tests successful/) {
+		if( $self->debug ) { # from H.Merijn Brand
+			my $prove = File::Spec->catfile(
+				dirname( $self->get_perl ),
+				'prove'
+				);
+
+			if( -x $prove ) {
+				my $prove_out =
+					join "\n\n",
+					map { scalar qx"$prove -wvb $_" }
+                    ($tests =~ m{^(t/\w+\.t)\s+[0-9]+}gm);
+				$prove_out =~ s/^.*\r//gm;
+				$self->_warn( $prove_out );
+				}
+			elsif( $self->debug ) {
+				$self->_print( "prove [$prove] was not executable!" );
+				}
+			}
+
+          $self->_die( "\nERROR: Tests failed!\n$tests\n\nAborting release\n" )
+          }
 
 	$self->_print( "all tests pass\n" );
 	}
@@ -859,22 +888,17 @@ sub dist
 	my $self = shift;
 	$self->_print( "Making dist... " );
 
-	unless( -e $self->{'Makefile'} )
-		{
-		$self->_debug( "no Makefle, so skipping" );
-		$self->_print( " no $self->{'Makefile'}---skipping\n" );
-		return;
-		}
+	$self->build_makefile;
 
 	my $messages = $self->run( "$self->{make} dist 2>&1 < $self->{devnull}" );
-	$self->_debug( "messages are [$messages]" );
+	$self->_debug( "messages are [$messages]\n" );
 
 	# If the distro isn't already set, try to guess it
 	unless( $self->local_file )
 		{
-		$self->_debug( ", guessing local distribution name" );
-		my( $guess ) = $messages =~ /^\s*gzip.+?\b'?(\S+\.tar)'?\s*$/m;
-		$self->_debug( "guessed [$guess]" );
+		$self->_debug( ", guessing local distribution name\n" );
+		my( $guess ) = $messages =~ /(?:\s|')(\S+\.tar)/;
+		$self->_debug( "guessed [$guess]\n" );
 		$self->local_file( $guess );
 
 		$self->_die( "Couldn't guess distname from dist output\n" )
@@ -1052,9 +1076,9 @@ deprecated.
 
 sub manifest_name { 'MANIFEST' }
 
-sub manifest { 
+sub manifest {
 	$_[0]->_warn( "manifest is deprecated. Use manifest_name" );
-	&manifest_name 
+	&manifest_name
 	}
 
 =item files_in_manifest
@@ -1066,14 +1090,14 @@ Return the filenames in the manifest file as a list.
 sub files_in_manifest
 	{
 	my $self = shift;
-	
+
 	require ExtUtils::Manifest;
-	
+
 	# I want to use ExtUtils::Manifest so it automatically
 	# follows the right MANIFEST rules, but I have to adapt
 	# it's output to my output. Annoying, for sure.
 	my $hash = do {
-		local $SIG{'__WARN__'} = sub { 
+		local $SIG{'__WARN__'} = sub {
 			my $message = shift;
 			if( $message =~ m/Debug: (.*)/ )
 				{
@@ -1084,10 +1108,10 @@ sub files_in_manifest
 				$self->_die( "files_in_manifest: could not open file\n" );
 				}
 			};
-			
+
 		ExtUtils::Manifest::maniread( $self->manifest_name );
 		};
-	
+
 	sort keys %$hash;
 	}
 
@@ -1142,7 +1166,7 @@ Returns the number of files which it successfully touched.
 sub touch
 	{
 	my( $self, @files ) = @_;
-	
+
 	my $time = time;
 
 	my $count = 0;
@@ -1197,9 +1221,9 @@ sub check_for_passwords
 	{
 	if( my $pass = $_[0]->config->cpan_user && $_[0]->get_env_var( "CPAN_PASS" )  )
 		{
-		$_[0]->config->set( 'cpan_pass', $pass ); 
+		$_[0]->config->set( 'cpan_pass', $pass );
 		}
-		
+
 	$_[0]->_debug( "CPAN pass is " . $_[0]->config->cpan_pass );
 	}
 
@@ -1269,7 +1293,7 @@ sub run
 
 	$self->_debug( "$command\n" );
 	$self->_die( "Didn't get a command!" ) unless defined $command;
-	
+
 	open my($fh), "$command |" or $self->_die( "Could not open command [$command]: $!" );
 	$fh->autoflush;
 
@@ -1382,10 +1406,12 @@ module form.
 Andy Lester handled the maintenance while I was on my Big Camping
 Trip. He applied patches from many authors.
 
-Andreas KE<ouml>nig suggested changes to make it work better with PAUSE.
+Andreas König suggested changes to make it work better with PAUSE.
 
 Chris Nandor helped with figuring out the broken SourceForge stuff.
 
+H.Merijn Brand has contributed many patches and features.
+
 =head1 SOURCE AVAILABILITY
 
 This source is in Github:
@@ -1398,7 +1424,7 @@ brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2002-2009 brian d foy.  All rights reserved.
+Copyright (c) 2002-2011 brian d foy.  All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -1406,5 +1432,3 @@ it under the same terms as Perl itself.
 =cut
 
 1;
-
-__END__
